"""
This directory contains all manipulations.
One manipulation per file.
The file describes what is needed for the manipulation to run.
Things such as the number of tracks needed and extra parameters.
"""
